package availability

import (
	"errors"
	"time"
)

type DeadlineEnforcer struct {
	Action func(success chan<- struct{}, terminate <-chan struct{})
}

func (deadlineEnforcer DeadlineEnforcer) DoWithin(timeoutMillis time.Duration) error {
	success := make(chan struct{})
	terminate := make(chan struct{})
	go deadlineEnforcer.Action(success, terminate)
	select {
	case <-success:
		return nil
	case <-time.After(timeoutMillis):
		close(terminate)
		return errors.New("timeout")
	}
}
